@@ -253,7 +253,7 @@ from xml.sax.handler import ContentHandler
 from xml.sax.saxutils import escape, quoteattr
 from xml.dom.minidom import parseString
 
-__version__ = "0.9.5"
+__version__ = "0.9.6"
 
 __all__ = ['SolrException', 'Solr', 'SolrConnection',
            'Response', 'SearchHandler']
@@ -282,7 +282,7 @@ class SolrException(Exception):
     tracebacks from the Java runtime.
     """
 
-    def __init__(self, httpcode, reason=None, body=None):
+    def __init__(self, httpcode=None, reason=None, body=None):
         self.httpcode = httpcode
         self.reason = reason
         self.body = body
@@ -807,13 +807,12 @@ class SearchHandler(object):
         """
         # Clean up optional parameters to match SOLR spec.
         query = []
-        to_str = lambda s: s.encode('utf-8') if isinstance(s, unicode) else s
         for key, value in params.items():
             key = key.replace(self.arg_separator, '.')
             if isinstance(value, (list, tuple)):
-                query.extend([(key, to_str(v)) for v in value])
+                query.extend([(key, strify(v)) for v in value])
             else:
-                query.append((key, to_str(value)))
+                query.append((key, strify(value)))
         request = urllib.urlencode(query, doseq=True)
         conn = self.conn
         if conn.debug:
@@ -831,6 +830,12 @@ class SearchHandler(object):
         return data
 
 
+def strify(s):
+    if isinstance(s, unicode):
+        return s.encode('utf-8')
+    else:
+        return s
+
 # ===================================================================
 # Response objects
 # ===================================================================
