@@ -7,6 +7,8 @@ Meant to be run against Solr 1.2+.
 """
 
 # stdlib
+import cPickle
+import pickle
 import socket
 import datetime
 import unittest
@@ -28,7 +30,7 @@ SOLR_HTTPS = "https://" + SOLR_HOST + ":" + SOLR_PORT_HTTPS + SOLR_PATH
 
 
 def get_rand_string():
-    return "".join(choice(digits)  for x in range(12))
+    return "".join(choice(digits) for x in range(12))
 
 
 def get_rand_userdoc(id=None, user_id=None, data=None):
@@ -95,7 +97,7 @@ class SolrConnectionTestCase(unittest.TestCase):
             results = self.query(conn, "id:" + doc["id"]).results
             self.assertEquals(
                 len(results), 0,
-                "Document (id:%s) should've been deleted" % doc["id"])
+                "Document (id:%s) should have been deleted" % doc["id"])
 
 
 class SolrBased(SolrConnectionTestCase):
@@ -1752,5 +1754,127 @@ class TestSolrRetries(SolrBased, TestRetries):
     pass
 
 
+class SolrExceptionHttpStatusPickleTestCase(unittest.TestCase):
+
+    module = pickle
+
+    def setUp(self):
+        self.initial = solr.SolrException(404, "Not Found")
+
+    # These tests check current constructions:
+
+    def test_unpicklable0(self):
+        self._test_unpickle(self.module.dumps(self.initial, protocol=0))
+
+    def test_unpicklable1(self):
+        self._test_unpickle(self.module.dumps(self.initial, protocol=1))
+
+    def test_unpicklable2(self):
+        self._test_unpickle(self.module.dumps(self.initial, protocol=2))
+
+    # These tests check legacy constructions, unpickled by self.module.
+    # The initial pickles were constructed with past releases of solrpy.
+    # (Persistent instances are known to exist in databases.)
+
+    def test_legacy_cPickle_0(self):
+        self._test_unpickle(
+            "csolr.core\nSolrException\np1\n(tRp2\n(dp3\nS'body'\np4\nNs"
+            "S'reason'\np5\nS'Not Found'\np6\nsS'httpcode'\np7\nI404\nsb.")
+
+    def test_legacy_cPickle_1(self):
+        self._test_unpickle(
+            'csolr.core\nSolrException\nq\x01)Rq\x02}q\x03(U\x04bodyq\x04NU'
+            '\x06reasonq\x05U\tNot Foundq\x06U\x08httpcodeq\x07M\x94\x01ub.')
+
+    def test_legacy_cPickle_2(self):
+        self._test_unpickle(
+            '\x80\x02csolr.core\nSolrException\nq\x01)Rq\x02}q\x03(U\x04body'
+            'q\x04NU\x06reasonq\x05U\tNot Foundq\x06U\x08httpcodeq\x07M\x94'
+            '\x01ub.')
+
+    def test_legacy_pickle_0(self):
+        self._test_unpickle(
+            "csolr.core\nSolrException\np0\n(tRp1\n(dp2\nS'body'\np3\nNs"
+            "S'reason'\np4\nS'Not Found'\np5\nsS'httpcode'\np6\nI404\nsb.")
+
+    def test_legacy_pickle_1(self):
+        self._test_unpickle(
+            'csolr.core\nSolrException\nq\x00)Rq\x01}q\x02(U\x04bodyq\x03NU'
+            '\x06reasonq\x04U\tNot Foundq\x05U\x08httpcodeq\x06M\x94\x01ub.')
+
+    def test_legacy_pickle_2(self):
+        self._test_unpickle(
+            '\x80\x02csolr.core\nSolrException\nq\x00)Rq\x01}q\x02(U\x04body'
+            'q\x03NU\x06reasonq\x04U\tNot Foundq\x05U\x08httpcodeq\x06M\x94'
+            '\x01ub.')
+
+    def _test_unpickle(self, s):
+        loaded = self.module.loads(s)
+        self.assertEqual(loaded.httpcode, self.initial.httpcode)
+        self.assertEqual(loaded.reason, self.initial.reason)
+        self.assertEqual(loaded.body, self.initial.body)
+        self.assertEqual(repr(loaded), repr(self.initial))
+        self.assertEqual(str(loaded), str(self.initial))
+
+
+class SolrExceptionSimpleMessagePickleTestCase(
+    SolrExceptionHttpStatusPickleTestCase):
+
+    def setUp(self):
+        # Doesn't really fit the apparent intention for SolrException,
+        # but this is used and has been used in the past, so let's
+        # continue to work with existing pickled exceptions.
+        self.initial = solr.SolrException("Simple message, not HTTP status")
+
+    def test_legacy_cPickle_0(self):
+        self._test_unpickle(
+            "csolr.core\nSolrException\np1\n(tRp2\n(dp3\nS'body'\np4\nNs"
+            "S'reason'\np5\nNsS'httpcode'\np6\n"
+            "S'Simple message, not HTTP status'\np7\nsb.")
+
+    def test_legacy_cPickle_1(self):
+        self._test_unpickle(
+            'csolr.core\nSolrException\nq\x01)Rq\x02}q\x03(U\x04bodyq\x04NU'
+            '\x06reasonq\x05NU\x08httpcodeq\x06U'
+            '\x1fSimple message, not HTTP statusq\x07ub.')
+
+    def test_legacy_cPickle_2(self):
+        self._test_unpickle(
+            '\x80\x02csolr.core\nSolrException\nq\x01)Rq\x02}q\x03(U\x04body'
+            'q\x04NU\x06reasonq\x05NU\x08httpcodeq\x06U'
+            '\x1fSimple message, not HTTP statusq\x07ub.')
+
+    def test_legacy_pickle_0(self):
+        self._test_unpickle(
+            "csolr.core\nSolrException\np0\n(tRp1\n(dp2\nS'body'\np3\nNs"
+            "S'reason'\np4\nNsS'httpcode'\np5\n"
+            "S'Simple message, not HTTP status'\np6\nsb.")
+
+    def test_legacy_pickle_1(self):
+        self._test_unpickle(
+            'csolr.core\nSolrException\nq\x00)Rq\x01}q\x02(U\x04bodyq\x03NU'
+            '\x06reasonq\x04NU\x08httpcodeq\x05U'
+            '\x1fSimple message, not HTTP statusq\x06ub.')
+
+    def test_legacy_pickle_2(self):
+        self._test_unpickle(
+            '\x80\x02csolr.core\nSolrException\nq\x00)Rq\x01}q\x02(U\x04body'
+            'q\x03NU\x06reasonq\x04NU\x08httpcodeq\x05U'
+            '\x1fSimple message, not HTTP statusq\x06ub.')
+
+
+class SolrExceptionHttpStatusCPickleTestCase(
+    SolrExceptionHttpStatusPickleTestCase):
+
+    module = cPickle
+
+
+class SolrExceptionSimpleMessageCPickleTestCase(
+    SolrExceptionSimpleMessagePickleTestCase):
+
+    module = cPickle
+
+
+
 if __name__ == "__main__":
     unittest.main()
